@@ -1,5 +1,6 @@
 /*                                                         -*- c++ -*-
- * Copyright (C) 2006 - 2019 Andreas Persson
+ * Copyright (C) 2006 - 2020 Andreas Persson
+ * Copyright (C) 2007 - 2025 Christian Schoenebeck
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -71,10 +72,10 @@
 
 class MainWindow;
 
-class PropDialog : public ManagedWindow,
-                   public PropEditor<DLS::Info> {
+class FilePropDialog : public ManagedWindow,
+                       public PropEditor<DLS::Info> {
 public:
-    PropDialog();
+    FilePropDialog();
     void set_file(gig::File* file);
 
     // implementation for abstract methods of interface class "ManagedWindow"
@@ -135,28 +136,109 @@ protected:
     void set_MIDIProgram(uint32_t value);
 
     sigc::signal<void> sig_name_changed;
-    VBox vbox;
+    Gtk::Notebook tabs;
+    VBox vbox[3];
     HButtonBox buttonBox;
     Gtk::Button quitButton;
+
+    // tab 1
     Table table;
     StringEntry eName;
     BoolEntry eIsDrum;
     NumEntryTemp<uint16_t> eMIDIBank;
     NumEntryTemp<uint32_t> eMIDIProgram;
     NumEntryGain eAttenuation;
-    BoolEntryPlus6 eGainPlus6;
     NumEntryTemp<uint16_t> eEffectSend;
     NumEntryTemp<int16_t> eFineTune;
     NumEntryTemp<uint16_t> ePitchbendRange;
     BoolEntry ePianoReleaseMode;
     NoteEntry eDimensionKeyRangeLow;
     NoteEntry eDimensionKeyRangeHigh;
+
+    // tab 2
+    Table table2;
+    StringEntry eName2;
+    StringEntry eCreationDate;
+    StringEntryMultiLine eComments;
+    StringEntry eProduct;
+    StringEntry eCopyright;
+    StringEntry eArtists;
+    StringEntry eGenre;
+    StringEntry eKeywords;
+    StringEntry eEngineer;
+    StringEntry eTechnician;
+    StringEntry eSoftware;
+    StringEntry eMedium;
+    StringEntry eSource;
+    StringEntry eSourceForm;
+    StringEntry eCommissioned;
+    StringEntry eSubject;
+};
+
+class SampleProps : public ManagedWindow,
+                    public PropEditor<gig::Sample> {
+public:
+    SampleProps();
+    void set_sample(gig::Sample* sample);
+    gig::Sample* get_sample() { return m; }
+    void update_name();
+    sigc::signal<void>& signal_name_changed() {
+        return sig_name_changed;
+    }
+
+    // implementation for abstract methods of interface class "ManagedWindow"
+    virtual Settings::Property<int>* windowSettingX() { return &Settings::singleton()->samplePropsWindowX; }
+    virtual Settings::Property<int>* windowSettingY() { return &Settings::singleton()->samplePropsWindowY; }
+    virtual Settings::Property<int>* windowSettingWidth() { return &Settings::singleton()->samplePropsWindowW; }
+    virtual Settings::Property<int>* windowSettingHeight() { return &Settings::singleton()->samplePropsWindowH; }
+
+protected:
+    void set_Name(const gig::String& name);
+
+    sigc::signal<void> sig_name_changed;
+    Gtk::Notebook tabs;
+    VBox vbox[3];
+    HButtonBox buttonBox;
+    Gtk::Button quitButton;
+
+    // tab 1
+    Table table;
+    StringEntry eName;
+    NoteEntry eUnityNote;
+    ReadOnlyLabelWidget eSampleGroup;
+    ReadOnlyLabelWidget eSampleFormatInfo;
+    ReadOnlyLabelWidget eSampleID;
+    ReadOnlyLabelWidget eChecksum;
+    NumEntryTemp<uint32_t> eLoopsCount;
+    NumEntryTemp<uint32_t> eLoopStart;
+    NumEntryTemp<uint32_t> eLoopLength;
+    ChoiceEntry<gig::loop_type_t> eLoopType;
+    NumEntryTemp<uint32_t> eLoopPlayCount;
+    // tab 2
+    Table table2;
+    StringEntry eName2;
+    StringEntry eCreationDate;
+    StringEntryMultiLine eComments;
+    StringEntry eProduct;
+    StringEntry eCopyright;
+    StringEntry eArtists;
+    StringEntry eGenre;
+    StringEntry eKeywords;
+    StringEntry eEngineer;
+    StringEntry eTechnician;
+    StringEntry eSoftware;
+    StringEntry eMedium;
+    StringEntry eSource;
+    StringEntry eSourceForm;
+    StringEntry eCommissioned;
+    StringEntry eSubject;
 };
 
 class ProgressDialog : public Gtk::Dialog {
 public:
     ProgressDialog(const Glib::ustring& title, Gtk::Window& parent);
     void set_fraction(float fraction) { progressBar.set_fraction(fraction); }
+    void set_activity(const char* txt);
 protected:
     Gtk::ProgressBar progressBar;
 };
@@ -167,8 +249,9 @@ public:
     Glib::Dispatcher& signal_progress();
     Glib::Dispatcher& signal_finished(); ///< Finished successfully, without error.
     Glib::Dispatcher& signal_error();
-    void progress_callback(float fraction);
+    void progress_callback(float fraction, const char* activity);
     float get_progress();
+    const char* get_activity() const { return activity; }
     void join();
     const Glib::ustring filename;
     Glib::ustring error_message;
@@ -191,6 +274,7 @@ private:
     Glib::Dispatcher progress_dispatcher;
     Glib::Dispatcher error_dispatcher;
     float progress;
+    const char* activity;
 };
 
 class Loader : public LoaderSaverBase {
@@ -272,6 +356,8 @@ protected:
     Glib::RefPtr<Gio::SimpleAction> m_actionEditScript;
     Glib::RefPtr<Gio::SimpleAction> m_actionRemoveScript;
 
+    Glib::RefPtr<Gio::SimpleAction> m_actionInstrDoubleClickOpensProps;
+
     Glib::RefPtr<Gio::SimpleAction> m_actionToggleCopySampleUnity;
     Glib::RefPtr<Gio::SimpleAction> m_actionToggleCopySampleTune;
     Glib::RefPtr<Gio::SimpleAction> m_actionToggleCopySampleLoop;
@@ -291,8 +377,9 @@ protected:
     RegionChooser m_RegionChooser;
     DimRegionChooser m_DimRegionChooser;
 
-    PropDialog propDialog;
+    FilePropDialog fileProps;
     InstrumentProps instrumentProps;
+    SampleProps sampleProps;
     MidiRules midiRules;
 
     /**
@@ -327,6 +414,7 @@ protected:
 #if !USE_GTKMM_BUILDER
     void on_instrument_selection_change(Gtk::RadioMenuItem* item);
 #endif
+    void on_action_move_instr();
     void on_sel_change();
     void region_changed();
     void dimreg_changed();
@@ -352,9 +440,9 @@ protected:
     void add_region_to_dimregs(gig::Region* region, bool stereo, bool all_dimregs);
     void update_dimregs();
 
-    class ModelColumns : public Gtk::TreeModel::ColumnRecord {
+    class InstrumentsModel : public Gtk::TreeModel::ColumnRecord {
     public:
-        ModelColumns() {
+        InstrumentsModel() {
           add(m_col_nr);
           add(m_col_name);
           add(m_col_instr);
@@ -367,16 +455,16 @@ protected:
         Gtk::TreeModelColumn<gig::Instrument*> m_col_instr;
         Gtk::TreeModelColumn<Glib::ustring> m_col_scripts;
         Gtk::TreeModelColumn<Glib::ustring> m_col_tooltip;
-    } m_Columns;
+    } m_InstrumentsModel;
 
     VBox m_VBox;
     Gtk::HPaned m_HPaned;
 
     Gtk::ScrolledWindow m_ScrolledWindow;
 
-    Gtk::TreeView m_TreeView;
-    Glib::RefPtr<Gtk::ListStore> m_refTreeModel;
-    Glib::RefPtr<Gtk::TreeModelFilter> m_refTreeModelFilter; //FIXME: I really would love to get rid of TreeModelFilter, because it causes behavior conflicts with get_model() all over the place (see the respective comments regarding get_model()), however I found no other way to filter a treeview effectively.
+    Gtk::TreeView m_TreeViewInstruments;
+    Glib::RefPtr<Gtk::ListStore> m_refInstrumentsTreeModel;
+    Glib::RefPtr<Gtk::TreeModelFilter> m_refInstrumentsModelFilter; //FIXME: I really would love to get rid of TreeModelFilter, because it causes behavior conflicts with get_model() all over the place (see the respective comments regarding get_model()), however I found no other way to filter a treeview effectively.
 
 #if USE_GTKMM_BUILDER
     Gtk::Menu* menuMacro;
@@ -486,10 +574,14 @@ protected:
     void on_action_quit();
     void show_instr_props();
     bool instr_props_set_instrument();
+    void show_sample_props();
+    bool sample_props_set_sample();
+    void sample_name_changed_by_sample_props(Gtk::TreeModel::iterator& it);
     void show_midi_rules();
     void show_script_slots();
     void on_action_view_status_bar();
     void on_auto_restore_win_dim();
+    void on_instr_double_click_opens_props();
     void on_save_with_temporary_file();
     void on_action_refresh_all();
     void on_action_warn_user_on_extensions();
@@ -524,6 +616,7 @@ protected:
     void on_action_add_script_group();
     void on_action_add_script();
     void on_action_edit_script();
+    void editScript(gig::Script* script);
     void on_action_remove_script();
 
     void on_action_add_instrument();
